
-- iSteg - applescript outguess frontend
-- concept and p.o.c. code by Hany El Imam (hany@hanynet.com)
-- code by Jonathan Nathan


property image_file_types : {"public.jpeg", "public.gif", "public.tiff", "public.png", "GIF ", "JPE ", "JPEG", "JPG ", "MAC ", "PCT ", "PIC ", "PICT", "PNG ", "PNGf", "TIF ", "TIFF"}
property text_file_types : {"public.plain-text", "TEXT", "text", "txt", "zip"}
property first_run : true

on awake from nib the_object
	set object_name to name of the_object as Unicode text
	if object_name = "main" then
		my register_settings()
		my read_settings()
		tell the_object
			if first_run then center
			set first_run to false
			my write_some_settings({"first_run"}, {first_run})
			tell text field "source_picture_file" to register drag types {"file names"}
			tell text field "source_text_file" to register drag types {"file names"}
			show
		end tell
		
	end if
end awake from nib

on clicked the_object
	set object_name to name of the_object as Unicode text
	
	if object_name = "max_keys_default" then
		tell window "main"
			set {should_enable, the_value} to {false, ""}
			if (state of the_object = 0) then set {should_enable, the_value} to {true, integer value of slider "max_keys_slider"}
			set enabled of slider "max_keys_slider" to should_enable
			set contents of text field "max_keys_text_field" to the_value
		end tell
		
	else if object_name = "iterator_seed_limit_default" then
		tell window "main"
			set {should_enable, the_value} to {false, ""}
			if (state of the_object = 0) then set {should_enable, the_value} to {true, integer value of slider "iterator_seed_limit_slider"}
			set enabled of slider "iterator_seed_limit_slider" to should_enable
			set contents of text field "iterator_seed_limit_text_field" to the_value
		end tell
		
	else if object_name = "paypaldonate" then
		do shell script "open http://www.hanynet.com/donate.html"
		
	else if object_name = "edit_source_text_file" then
		do shell script "open " & quoted form of (get string value of text field "source_text_file" of window "main")
		
	else if object_name = "choose_source_picture_file" then
		tell window "main"
			set string value of text field "source_picture_file" to (choose file of type image_file_types with prompt "Choose a source picture file:" without invisibles)
			set enabled of button "clear_source_picture_file" to true
		end tell
		
	else if object_name = "choose_source_text_file" then
		tell window "main"
			set string value of text field "source_text_file" to (choose file of type text_file_types with prompt "Choose a source text file:" without invisibles)
			set enabled of button "edit_source_text_file" to true
			set enabled of button "clear_source_text_file" to true
		end tell
		
	else if object_name = "close" then
		close panel (window of the_object)
		
	else if object_name = "clear_source_text_file" then
		tell window "main"
			set contents of text field "source_text_file" to ""
			set enabled of button "edit_source_text_file" to false
			set enabled of button "clear_source_text_file" to false
		end tell
		
	else if object_name = "clear_source_picture_file" then
		tell window "main"
			set contents of text field "source_picture_file" to ""
			set enabled of button "clear_source_picture_file" to false
		end tell
		
	else if object_name = "make_password" then
		set contents of text field "password" of window "main" to make_password(10)
		
	else if object_name = "help" then
		display window "help" attached to window "main"
		
	else if object_name = "docs" then
		set docs_path to POSIX path of ((path to me as string) & "Contents:Resources:")
		do shell script "open -a Preview \"" & (docs_path) & "iSteg Docs.pdf\""
		
	else if object_name = "encode" then
		set should_enable to (state of the_object = 1)
		tell window "main"
			set enabled of text field "source_text_file" to should_enable
			set enabled of button "choose_source_text_file" to should_enable
			if should_enable then
				set enabled of button "clear_source_text_file" to ((get string value of text field "source_text_file")  "")
				set enabled of button "edit_source_text_file" to ((get string value of text field "source_text_file")  "")
			else
				set enabled of button "clear_source_text_file" to false
				set enabled of button "edit_source_text_file" to false
			end if
			
			set enabled of slider "jpeg_quality_slider" to should_enable
			set enabled of button "max_keys_default" to should_enable
			set enabled of button "iterator_seed_limit_default" to should_enable
			
			if should_enable then
				set enabled of slider "max_keys_slider" to ((get state of button "max_keys_default") = 0)
				set enabled of slider "iterator_seed_limit_slider" to ((get state of button "iterator_seed_limit_default") = 0)
			else
				set enabled of slider "max_keys_slider" to false
				set enabled of slider "iterator_seed_limit_slider" to false
			end if
		end tell
		
	else if object_name = "process" then
		set outguess_path to quoted form of POSIX path of ((path to me as Unicode text) & "Contents:Resources:outguess")
		
		tell window "main"
			set user_password to (string value of text field "password")
			if user_password  "" then set user_password to " -k " & quoted form of user_password
			
			set do_encode to (state of button "encode" = 1)
			
			set source_text_file to (string value of text field "source_text_file")
			set source_picture_file to (string value of text field "source_picture_file")
			
			set jpeg_quality to " -p " & (string value of text field "jpeg_quality_text_field")
			
			set max_keys to ""
			if (state of button "max_keys_default" = 0) then set max_keys to (" -x " & (string value of text field "max_keys_text_field"))
			
			set iterator_seed_limit to ""
			if (state of button "iterator_seed_limit_default" = 0) then set iterator_seed_limit to (" -i " & (string value of text field "iterator_seed_limit_text_field"))
			
			set preserve_statistics to " -F +"
			if (state of button "preserve_statistics" = 0) then set preserve_statistics to " -F -"
			
			set error_correction to ""
			if (state of button "error_correction" = 1) then set error_correction to " -e"
			
			set extended_feedback to ""
			if (state of button "extended_feedback" = 1) then set extended_feedback to " -tt"
		end tell
		
		try
			if source_picture_file = "" then error "an input picture"
			if do_encode then
				if source_text_file = "" then error "a source text"
				set outfile to POSIX path of (choose file name with prompt "Save the encoded file as:" default name "output.jpg")
				set the_script to outguess_path & preserve_statistics & iterator_seed_limit & max_keys & extended_feedback & jpeg_quality & error_correction & user_password & " -d " & quoted form of source_text_file & " " & quoted form of source_picture_file & " " & quoted form of outfile & " 2>&1"
			else
				set outfile to POSIX path of (choose file name with prompt "Save the decoded file as:" default name "output.txt")
				set the_script to outguess_path & preserve_statistics & error_correction & user_password & " -r " & quoted form of source_picture_file & " " & quoted form of outfile & " 2>&1"
			end if
		on error the_error number error_number
			if error_number = -128 then return
			beep
			activate
			return display dialog "iSteg Error" & return & return & "You must specify " & the_error & " file." buttons {"OK"} default button 1 with icon 0 giving up after 20 attached to window "main"
		end try
		
		tell progress indicator "busy" of window "main"
			set visible to true
			start
		end tell
		tell window "finish"
			set contents of text view "the_script" of scroll view "the_script" to the_script
			try
				set contents of text view "output" of scroll view "output" to (do shell script the_script)
			on error the_error
				set contents of text view "output" of scroll view "output" to the_error
			end try
		end tell
		tell progress indicator "busy" of window "main"
			stop
			set visible to false
		end tell
		display window "finish" attached to window "main"
		
	end if
end clicked

on action the_object
	set object_name to name of the_object as Unicode text
	tell window "main"
		set the_value to integer value of the_object
		
		if object_name = "jpeg_quality_slider" then
			set contents of text field "jpeg_quality_text_field" to the_value
			
		else if object_name = "max_keys_slider" then
			set contents of text field "max_keys_text_field" to the_value
			
		else if object_name = "iterator_seed_limit_slider" then
			set contents of text field "iterator_seed_limit_text_field" to the_value
			
		end if
	end tell
end action

on conclude drop the_object drag info drag_info
	--do nothing
end conclude drop

on drop the_object drag info drag_info
	set object_name to name of the_object as Unicode text
	
	set data_types to types of pasteboard of drag_info
	if "file names" is in data_types then
		set preferred type of pasteboard of drag_info to "file names"
		set dropped_files to contents of pasteboard of drag_info
		set dropped_file to item 1 of dropped_files
		if dropped_file ends with "/" then return
		set the_info to info for (dropped_file as POSIX file)
		if object_name = "source_picture_file" then
			set file_types to image_file_types
		else
			set file_types to text_file_types
		end if
		if the_info's type identifier is in file_types then
			set string value of the_object to dropped_file
			tell window "main"
				if (state of button "encode" = 1) then
					set enabled of button ("clear_" & object_name) to true
					try
						set enabled of button ("edit_" & object_name) to true
					end try
				end if
			end tell
		end if
	end if
	set preferred type of pasteboard of drag_info to ""
end drop

on should quit after last window closed the_object
	return true
end should quit after last window closed

on register_settings()
	set {the_settings, the_setting_names} to my get_settings()
	tell user defaults
		repeat with i from 1 to (count the_setting_names)
			try
				make new default entry at end of default entries with properties {name:the_setting_names's item i, contents:the_settings's item i}
			end try
		end repeat
	end tell
end register_settings

on read_settings()
	set {the_settings, the_setting_names} to my get_settings()
	set the_settings to {}
	tell user defaults
		repeat with i from 1 to (count the_setting_names)
			try
				set end of the_settings to contents of default entry (the_setting_names's item i)
			end try
		end repeat
	end tell
	my assign_settings(the_settings)
end read_settings

on write_some_settings(these_setting_names, these_setting_values)
	tell user defaults
		repeat with i from 1 to (count these_setting_names)
			try
				set contents of default entry (these_setting_names's item i) to (these_setting_values's item i)
			end try
		end repeat
		try
			synchronize
		end try
	end tell
end write_some_settings

on assign_settings(the_settings)
	set {first_run} to the_settings
end assign_settings

on get_settings()
	set the_settings to {first_run}
	set the_setting_names to {"first_run"}
	return {the_settings, the_setting_names}
end get_settings

property numerics : (characters of "0123456789")
property letters_vowels : (characters of "aeiou")
property letters_consonants : (characters of "bdfghjklmnprstvwz")
property letters_consonants2 : (characters of "bdfglmnprstvxz")
property start_consonants : letters_consonants & {"ch", "th", "sh"}
property end_consonants : letters_consonants2 & {"ck", "tch", "th", "sh"}

on make_password(password_length)
	set the_password to (my make_phoneme() & my make_phoneme() & (some item of numerics))
	repeat while ((length of the_password) < password_length)
		set the_password to the_password & (some item of numerics)
	end repeat
	return the_password
end make_password

on make_phoneme()
	return (some item of start_consonants) & (some item of letters_vowels) & (some item of end_consonants)
end make_phoneme